local L = LibStub("AceLocale-3.0"):GetLocale("Broker_AddonControl")
Broker_AddonControlUI = {}

function Broker_AddonControlUI.CreateConfigMenu()
	local configPanel = CreateFrame("Frame", "Broker_AddonControlConfigFrame", UIParent)
	configPanel.name = "Broker_AddonControl"
	configPanel.okay = function (self) return end
	configPanel.cancel = function (self) return end

	local addonName, addonTitle, addonNotes = GetAddOnInfo('Broker_AddonControl')
	local configPanelText = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
	configPanelText:SetPoint('TOPLEFT', 16, -16)
	configPanelText:SetText(addonTitle)

	local configPanelDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')
	--configPanelDesc:SetHeight(32)
	configPanelDesc:SetPoint('TOPLEFT', configPanelText, 'BOTTOMLEFT', 0, -8)
	configPanelDesc:SetPoint('RIGHT', configPanel, -32, 0)
	configPanelDesc:SetNonSpaceWrap(true)
	configPanelDesc:SetJustifyH('LEFT')
	configPanelDesc:SetJustifyV('TOP')
	configPanelDesc:SetText(addonNotes)
	
	local altReload = Broker_AddonControlUI.CreateCheckButton(L["ALT ReloadUI"], configPanel ,L["If ALT must be held to reload UI."], "altReloadUI",'InterfaceOptionsCheckButtonTemplate')
	altReload:SetPoint('TOPLEFT', configPanelDesc, 'BOTTOMLEFT', 2, -10)

	local maxGroupCharDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	maxGroupCharDesc:SetPoint('TOPLEFT', altReload, 'BOTTOMLEFT', -10, -10)
	maxGroupCharDesc:SetText(L["Start Group Characters"])

	local maxCharGroup = CreateFrame("Slider", configPanel:GetName() .. "maxCharGroup", configPanel, "OptionsSliderTemplate")
	maxCharGroup:SetMinMaxValues(1, 30)
	maxCharGroup:SetValueStep(1)
	maxCharGroup:SetValue(Broker_AddonControl.GetConfigValue("maxMatchCharacters"))
	_G[maxCharGroup:GetName() .. "High"]:SetText("30")
	_G[maxCharGroup:GetName() .. "Low"]:SetText("1")
	maxCharGroup:SetScript("OnValueChanged", Broker_AddonControlUI.BarMatchCharsChanged)
	maxCharGroup:SetPoint('TOPLEFT', maxGroupCharDesc, 'BOTTOMLEFT', 10, 0)
	maxCharGroup:SetScript('OnEnter', function(self)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:SetText(L["The amount of chars to compare at the start of each addon to group the addons together. Recommended to set it to 4, meaning if the first 4 chars of an addon NAME match, they will be grouped together."], nil, nil, nil, 1, 1)
		GameTooltip:Show() end)
	maxCharGroup:SetScript('OnLeave', function(self) GameTooltip:Hide() end)	
	
	Broker_AddonControl.maxCharGroupValue = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	Broker_AddonControl.maxCharGroupValue:SetPoint('TOPLEFT', maxCharGroup, 'TOPRIGHT', 10, -2)
	Broker_AddonControl.maxCharGroupValue:SetText(string.format("%.1f", Broker_AddonControl.GetConfigValue("maxMatchCharacters")))	
	
	local hardCoded = Broker_AddonControlUI.CreateCheckButton(L["Ignore hard coded groups"], configPanel ,L["Hard coded groups will not be used (Only grouping by Start Group Chars value)."], "ignoreHardCodedGroups",'InterfaceOptionsCheckButtonTemplate',Broker_AddonControl.CreateAddonGroups)
	hardCoded:SetPoint('TOPLEFT', Broker_AddonControl.maxCharGroupValue, 'TOPRIGHT', 10, 6)	
	
	local hideNotes = Broker_AddonControlUI.CreateCheckButton(L["Hide notes"], configPanel ,L["Hide the displayed notes in tooltip."], "hideNotes",'InterfaceOptionsCheckButtonTemplate')
	hideNotes:SetPoint('TOPLEFT', maxCharGroup, 'BOTTOMLEFT', -10, -10)
	
	local clear = CreateFrame("Button", "AddonCTRLPurgeHiddenList", configPanel, "UIPanelButtonTemplate2")
	clear:SetText(L["Purge Hidden List"])
	clear:SetWidth(160)
	clear:SetPoint('TOPLEFT', hideNotes, 'BOTTOMLEFT', 0, -10)
	clear:SetScript("OnClick", function()
		Broker_AddonControl.PurgeHiddenList()
	end)
	clear.tooltipText = L["Purge addons that you don't have installed any more from hidden list."]
	
	InterfaceOptions_AddCategory(configPanel)
end

function Broker_AddonControlUI.BarMatchCharsChanged(self, value)
	--Broker_AddonControl.maxMatchCharacters = value
	Broker_AddonControl.SetConfigValue("maxMatchCharacters", value)
	Broker_AddonControl.maxCharGroupValue:SetText(string.format("%.1f", Broker_AddonControl.GetConfigValue("maxMatchCharacters")))			
	Broker_AddonControl.CreateAddonGroups();
end

function Broker_AddonControlUI.CreateCheckButton(name, parent, tooltipText, configKey, template, funct)
	local button = CreateFrame('CheckButton', parent:GetName() .. name, parent, template)
	_G[button:GetName() .. 'Text']:SetText(name)
	button:SetChecked(Broker_AddonControl.GetConfigValue(configKey))
	button:SetScript('OnClick', function(self)
		if self:GetChecked() then
			Broker_AddonControl.SetConfigValue(configKey, true)
		else 
			Broker_AddonControl.SetConfigValue(configKey, false)
		end
		if ( self.dependentControls ) then
			if ( self:GetChecked() ) then
				for _, control in pairs(self.dependentControls) do
					control:Enable()
				end
			else
				for _, control in pairs(self.dependentControls) do
					control:Disable()
				end
			end
		end
		if ( self.inverseDependentControls ) then
			if ( self:GetChecked() ) then
				for _, control in pairs(self.inverseDependentControls) do
					control:Disable()
				end
			else
				for _, control in pairs(self.inverseDependentControls) do
					control:Enable()
				end
			end
		end
		if (funct) then
			funct();
		end
	end)
	button:SetScript('OnEnter', function(self)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:SetText(tooltipText, nil, nil, nil, 1, 1)
		GameTooltip:Show() end)
	button:SetScript('OnLeave', function(self) GameTooltip:Hide() end)

	return button
end

function Broker_AddonControlUI.SetupDependentControl (dependency, control)
	if ( not dependency ) then
		return
	end
	
	assert(control)
	
	dependency.dependentControls = dependency.dependentControls or {}
	tinsert(dependency.dependentControls, control)

	if ( control.type ~= CONTROLTYPE_DROPDOWN ) then
		control.Disable = function (self) getmetatable(self).__index.Disable(self) _G[self:GetName().."Text"]:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b) end
		control.Enable = function (self) getmetatable(self).__index.Enable(self) _G[self:GetName().."Text"]:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b) end
		if dependency:GetChecked() then
			control:Enable()
		else
			control:Disable()
		end
	else
		control.Disable = function (self) UIDropDownMenu_DisableDropDown(self) end
		control.Enable = function (self) UIDropDownMenu_EnableDropDown(self) end
	end

end

function Broker_AddonControlUI.SetupInverseDependentControl (dependency, control)
	if ( not dependency ) then
		return
	end
	
	assert(control)
	
	dependency.inverseDependentControls = dependency.inverseDependentControls or {}
	tinsert(dependency.inverseDependentControls, control)

	if ( control.type ~= CONTROLTYPE_DROPDOWN ) then
		control.Disable = function (self) getmetatable(self).__index.Disable(self) _G[self:GetName().."Text"]:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b) end
		control.Enable = function (self) getmetatable(self).__index.Enable(self) _G[self:GetName().."Text"]:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b) end
		if dependency:GetChecked() then
			control:Disable()
		else
			control:Enable()
		end
	else
		control.Disable = function (self) UIDropDownMenu_DisableDropDown(self) end
		control.Enable = function (self) UIDropDownMenu_EnableDropDown(self) end
	end

end